{% extends "timepiece/business/base.html" %}
{% load timepiece_tags markup %}
{% load url from future %}

{% block title %}{{ business.name }}{% endblock %}

{% block crumbs %}
    {{ block.super }}
    <li><span class="divider">/</span> <a href="{% url 'view_business' business.pk %}">Business Detail</a></li>
{% endblock crumbs %}

{% block content %}
    <div class="row-fluid">
        <div class="span12">
            <h2>{{ business.name }}</h2>
            <ul class="nav nav-pills">
                {% if perms.timepiece.change_business %}
                    <li><a href='{% url 'edit_business' business.pk %}'>Edit business</a></li>
                    <li><a href="{% url 'delete_business' business.pk %}">Delete business</a></li>
                {% endif %}
            </ul>
        </div>
    </div>

    <div class="row-fluid">
        <div class="span12">
            <table class='table table-bordered table-striped'>
                <tr>
                    <th>Email:</th>
                    <td>{{ business.email }}</td>
                </tr>
                <tr>
                    <th>Description:</th>
                    <td>{{ business.description|linebreaks }}</td>
                </tr>
                <tr>
                    <th>Notes:</th>
                    <td>{{ business.notes|linebreaks }}</td>
                </tr>
            </table>
        </div>
    </div>
{% endblock %}
