{% extends 'timepiece/time-sheet/base.html' %}

{% block title %}Reject Entry{% endblock %}

{% block content %}
    <div class="row-fluid">
        <div class="span12">
            <h2>Reject Entry</h2>

            <br style="clear: both;" />

            <form class="form-inline" action="" method="post">
                {% csrf_token %}
                <h3>Are you sure you want to reject this entry?</h3>
                <input type="hidden" name="next" value="{{ next }}" />
                <div class="form-actions">
                    <input class="btn btn-primary" type="submit" name="Yes" value="Yes" />
                    <input class="btn" type="button" value="No" onclick="javascript:history.go(-1)" />
                </div>
            </form>
        </div>
    </div>
{% endblock %}
