{% extends "timepiece/base.html" %}
{% load url from future %}

{% block content %}
    {% block report_nav %}
        <div class="row-fluid">
            <div class="span12">
                <h2>Reports</h2>
                <ul class="nav nav-pills">
                    {% if perms.timepiece.view_entry_summary %}
                        <li{% ifequal active 'hourly_report' %} class="active"{% endifequal %}>
                            <a href="{% ifequal active 'hourly_report' %}#{% else %}{% url 'hourly_report' %}{% endifequal %}">Hourly</a>
                        </li>
                        <li{% ifequal active 'billable_hours' %} class="active"{% endifequal %}>
                            <a href="{% ifequal active 'billable_hours' %}#{% else %}{% url 'billable_hours' %}{% endifequal %}">Billable Hours</a>
                        </li>
                        <li{% ifequal active 'timepiece-summary' %} class="active"{% endifequal %}>
                            <a href="{% ifequal active 'timepiece-summary' %}#{% else %}{% url 'timepiece-summary' %}{% endifequal %}">General Ledger</a>
                        </li>
                    {% endif %}
                    {% if perms.timepiece.view_payroll_summary %}
                        <li{% ifequal active 'payroll_summary' %} class="active"{% endifequal %}>
                            <a href="{% ifequal active 'payroll_summary' %}#{% else %}{% url 'payroll_summary' %}{% endifequal %}">Payroll Summary</a>
                        </li>
                        <li{% ifequal active 'productivity_report' %} class="active"{% endifequal %}>
                            <a href="{% ifequal active 'productivity_report' %}#{% else %}{% url 'productivity_report' %}{% endifequal %}">Productivity</a>
                        </li>
                    {% endif %}
                </ul>
            </div>
        </div>
    {% endblock report_nav %}

    {% block report_content %}
    {% endblock report_content %}
{% endblock content %}
