{% extends "timepiece/base.html" %}
{% load url from future %}
{% load timepiece_tags %}

{% block title %}{{ action.capitalize }} Hours for Time Sheet{% endblock %}

{% block crumbs %}
    {{ block.super }}
    <li><span class="divider">/</span> <a href="{% url 'change_person_time_sheet' action timesheet_user.id from_date|date:"Y-m-d" %}">{{ action.capitalize }} Timesheet</a></li>
{% endblock crumbs %}

{% block content %}
    <div class="row-fluid">
        <div class="span12">
            <h2>{{ action.capitalize }} Hours for Time Sheet</h2>
            <h3>
                Are you sure you want to {{ action }} all of {{ timesheet_user.get_full_name }}'s timesheet,
                from {{ from_date|date:"N j, Y" }} to {{ to_date|date:"N j, Y" }}, which includes a total of {{ hours }} hours?
            </h3>
            {% if action == "approve" %}
                <p>After entries have been approved they are ready to be submitted for payroll.</p>
            {% else %}
                <p>After entries are verified they cannot be changed.</p>
            {% endif %}
        </div>
    </div>

    <div class="row-fluid">
        <div class="span12">
            <form method="post">
                {% csrf_token %}
                <input class="btn btn-primary" type="submit" name="do_action" value="Yes"/>
                <a href="{{ return_url }}"><input type="button" class="btn" value="No"/></a>
            </form>
        </div>
    </div>
{% endblock %}
