{% extends "timepiece/base.html" %}

{% block title %}Search Results{% endblock title %}

{% block content %}
    {% if form.errors %}
        {% for field in form %}
            {{ field.errors }}
        {% endfor %}
    {% endif %}
{% endblock content %}