{% extends 'timepiece/time-sheet/base.html' %}
{% load bootstrap_toolkit %}
{% load url from future %}

{% block crumbs %}
    {{ block.super }}
    <li><span class="divider">/</span> <a href="{% url 'timepiece-clock-out' entry.pk %}">Clock Out</a></li>
{% endblock crumbs %}

{% block title %}Clock Out of {{ entry.project.name }}{% endblock title %}

{% block content %}
    <div class="row-fluid">
        <div class="span12">
            <h2>Clock Out of {{ entry.activity.name }} on {{ entry.project.name }}</h2>

            <br style="clear: both;" />

            <form class="form-horizontal" action="" method="post" accept-charset="utf-8">
                {% csrf_token %}
                {{ form|as_bootstrap:"horizontal" }}
                <div class="form-actions">
                    <input class="btn btn-primary" type="submit" name="submit" value="Clock Out" />
                    <input class="btn" type="button" value="Cancel" onclick="window.location='{% url 'dashboard' %}'" />
                </div>
            </form>
        </div>
    </div>
{% endblock %}
