{% extends "timepiece/base.html" %}
{% load timepiece_tags %}
{% load url from future %}

{% block javascript %}
    {{ block.super }}
    <script src="{{ STATIC_URL }}timepiece/js/clear_form.js" type="text/javascript"></script>
{% endblock %}
