{% extends 'timepiece/time-sheet/base.html' %}
{% load url from future %}

{% block title %}Delete Entry{% endblock %}

{% block crumbs %}
    {{ block.super }}
    <li><span class="divider">/</span> <a href="{% url 'timepiece-delete' entry.pk %}">Delete Entry</a></li>
{% endblock crumbs %}

{% block content %}
    <div class="row-fluid">
        <div class="span12">
            <h2>Delete Entry</h2>

            <br style="clear: both;" />

            <form class="form-inline" action="{% url 'timepiece-delete' entry.id %}" method="post">
                {% csrf_token %}
                <h3>Are you sure you want to delete this entry?</h3>
                <input type="hidden" name="key" value="{{ entry.delete_key }}" />
                <div class="form-actions">
                    <input class="btn btn-primary" type="submit" value="Yes" />
                    <input class="btn" type="button" value="No" onclick="window.location='{% url 'dashboard' %}'" />
                </div>
            </form>
        </div>
    </div>
{% endblock %}
