{% extends "timepiece/time-sheet/base.html" %}

{% block title %}Reject Entries{% endblock title %}

{% block content %}
    <div class="row-fluid">
        <div class="span12">
            <h2>Reject Entries for {{ date|date:"F Y" }}</h2>

            <form class="form-inline" action="" method="post">
                {% csrf_token %}
                <h3>Are you sure you want to reject {{ user.first_name }}'s entries for {{ date|date:"F Y" }}?</h3>
                <input type="hidden" name="month" value="{{ date|date:'n' }}" />
                <input type="hidden" name="year" value="{{ date|date:'Y' }}" />
                <div class="form-actions">
                    <input class="btn btn-primary" type="submit" name="yes" value="Yes" />
                    <input class="btn" type="button" value="No" onclick="javascript:history.go(-1)" />
                </div>
            </form>
        </div>
    </div>
{% endblock content %}