{% extends "timepiece/base.html" %}

{% block title %}
    Delete {{ object }}
{% endblock title %}

{% block content %}
    <div class="row-fluid">
        <div class="span12">
            {% if object.get_full_name %}
                <h3>Are you sure you want to delete {{ object.get_full_name }}?</h3>
            {% else %}
                <h3>Are you sure you want to delete {{ object }}?</h3>
            {% endif %}
            <form method="post" action="">
                {% csrf_token %}
                <input type="hidden" name="delete" value="delete" />
                <input type="submit" class="btn btn-danger" value="Delete" />
                <input type="button" class="btn" value="Do Not Delete" onclick="history.back(-1)" />
            </form>
        </div>
    </div>
{% endblock content %}