{% extends "timepiece/base.html" %}
{% load url from future %}
{% load bootstrap_toolkit %}

{% block title %}My Settings{% endblock %}

{% block crumbs %}
    {{ block.super }}
    <li><span class="divider">/</span> <a href="{% url 'edit_settings' %}">My Settings</a></li>
{% endblock crumbs %}

{% block content %}
    <div class="row-fluid">
        <div class="span12">
            <h2>My Settings</h2>

            <br style="clear: both;" />

            <form class="form-horizontal" id="create-edit-person" action="" method="post" accept-charset="utf-8">
                {% csrf_token %}
                {{ user_form|as_bootstrap:"horizontal" }}
                {{ profile_form|as_bootstrap:"horizontal" }}
                <div class="form-actions">
                    <input class="btn btn-primary" type="submit" value="Save" name="submit">
                    <input class="btn" type="button" onclick="history.go(-1)" value="Cancel">
                </div>
            </form>
        </div>
    </div>
{% endblock %}
