{% extends "timepiece/project/base.html" %}
{% load url from future %}
{% load bootstrap_toolkit %}

{% block title %}Edit Project Relationship{% endblock %}

{% block crumbs %}
    {{ block.super }}
    <li><span class="divider">/</span> <a href="{% url 'edit_project_relationship' project.pk user.pk %}">Edit Relationship</a></li>
{% endblock crumbs %}

{% block content %}
    <div class="row-fluid">
        <div class="span12">
            <h2>Edit {{ user.get_full_name }}'s {{ project }} Relationship</h2>

            <br style="clear: both;" />

            <form class="form-horizontal" id="edit-project-relationship" action="" method="post" accept-charset="utf-8">
                {% csrf_token %}
                {{ relationship_form|as_bootstrap:"horizontal" }}
                <div class="form-actions">
                    <input class="btn btn-primary" type='submit' name='submit' value='Save' />
                    <input class="btn" type='button' value='Cancel' onclick='history.go(-1)' />
                </div>
            </form>
        </div>
    </div>
{% endblock %}
