{% extends "timepiece/base.html" %}
{% load url from future %}

{% block crumbs %}
    {{ block.super }}
    <li><span class="divider">/</span> <a href="{% url 'list_people' %}">People</a></li>
{% endblock crumbs %}
