{% extends 'timepiece/time-sheet/base.html' %}
{% load url from future %}

{% block crumbs %}
    {{ block.super }}
    <li><span class="divider">/</span> <a href="{% url 'invoice_projects' %}">Invoices</a></li>
{% endblock crumbs %}
