{% extends 'timepiece/time-sheet/invoice/base.html' %}
{% load timepiece_tags %}
{% load url from future %}

{% block title %}Invoice Hours for Time Sheet{% endblock %}

{% block crumbs %}
    {{ block.super }}
    <li><span class="divider">/</span> <a href="{% url 'delete_invoice' invoice.pk %}">Delete Invoice</a></li>
{% endblock crumbs %}

{% block content %}
    <div class="row-fluid">
        <div class="span12">
            <h2>
                Delete the invoice for {{ project.name }} hours {% if from_date %}from {{ from_date }} to {{ to_date }}{% else %}through {{ to_date }}{% endif %}
            </h2>
            <p>Deleting this invoice will return all associated entries to "approved" status. Are you sure you want to do this?</p>

            <form class="form-inline" method="post">{% csrf_token %}
                <input class="btn btn-primary" type='submit' name='delete' value='Delete Invoice' />
                <input class="btn" type='submit' value='Cancel' />
            </form>
        </div>
    </div>

<div class="row-fluid">
    <div class="span12">
        <h3>Selected Entries</h3>
        {% include 'timepiece/time-sheet/_weekly_entry_list.html' %}
    </div>
</div>
{% endblock %}
