{% extends 'timepiece/time-sheet/invoice/base.html' %}
{% load timepiece_tags %}
{% load url from future %}

{% block title %}Edit Invoice{% endblock %}

{% block crumbs %}
    {{ block.super }}
    <li><span class="divider">/</span> <a href="{% url 'edit_invoice' invoice.pk %}">Edit Invoice</a></li>
{% endblock crumbs %}

{% block content %}
    <div class="row-fluid">
        <div class="span12">
            {% invoice_subheaders 'edit' %}
        </div>
    </div>

    <div class="row-fluid">
        <div class="span12">
            <form class="form-horizontal" method="post">{% csrf_token %}
                {{ invoice_form.as_p }}
                <input class="btn" type="submit" name="submit" value="Update Invoice" />
                <a class="btn btn-danger" href="{% url 'delete_invoice' invoice.pk %}">Delete Invoice</a>
            </form>
        </div>
    </div>

    <div class="row-fluid">
        <div class="span12">
            <h3>Selected Entries</h3>
            {% if entries %}
                <table class="table table-bordered table-condensed">
                    <tr>
                        <th>Date</th>
                        <th>User</th>
                        <th>Time</th>
                        <th>Project</th>
                        <th>Activity</th>
                        <th>Hours</th>
                        <th>Comments</th>
                        <th>Remove</th>
                    </tr>
                    {% for entry in entries %}
                        {% ifchanged entry.start_time|date:"W" %}
                            <tr class="ledger_header">
                                <th colspan='8'>Week of {% week_start entry.start_time %} </th>
                            </tr>
                        {% endifchanged %}
                        <tr>
                            {% ifchanged entry.start_time|date %}
                                <td>{{ entry.start_time|date:"m/d/Y (D)" }}</td>
                            {% else %}
                                <td></td>
                            {% endifchanged %}
                        <td>{{ entry.user.get_full_name }} </td>
                        <td>{{ entry.start_time|date:"P" }} - {{ entry.end_time|date:"P" }} </td>
                        <td>{{ entry.project.name }}</td>
                        <td>{{ entry.activity.name }}</td>
                        <td class="hours">{{ entry.hours }}</td>
                        <td title="{{entry.comments}}">{{ entry.comments|truncatewords:12 }}</td>
                        <td><a href="{% url 'remove_invoice_entry' invoice.pk entry.pk %}">Remove</a></td>
                    </tr>
                    {% endfor %}
                </table>
            {% else %}
                <p>No entries were found</p>
            {% endif %}
        </div>
    </div>
{% endblock %}
